<?php

namespace App\Http\Controllers;

use App\Contact;
use App\Draft;
use Modules\Project\Entities\Project;

use App\Transaction;
use App\TransactionSellLine;
use DB;
use PDF;
use Illuminate\Http\Request;

class DraftController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(Request $request)
    {

        $title = "Drafts";

        $business_id = request()->session()->get('user.business_id');

        $contact_id = $request->contact_id;                              // get Contact Id From Request.

        $cards = Transaction::where('business_id',$business_id)->where('contact_id',$contact_id)->get();



        return view('drafts.create',compact('title','cards'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $rules = [

            'name' => 'required'
        ];

        $this->validate($request,$rules);

        $card = Draft::create([

            'name' => $request->name,
            'status' => "not_approve",
            'contact_id' => $request->contact_id,
            'cards_id' => json_encode($request->category)
        ]);


        if($card)

            return redirect()->route('contacts.show',$card->contact_id);

        return redirect()->route('drafts.create',$card->contact_id);

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Draft $draft)
    {

       
        
        $title = "Draft: ".$draft->name;   // title page.

        $total = 0;
      
      $projects = DB::select(DB::raw("SELECT pjt_projects.name  FROM pjt_projects"));
      $projects = json_decode(json_encode($projects));

      for ($i=0; $i < count($projects); $i++)
      {
          $projects[$i]->all = 0;
      }


        $res = "SELECT products.name ,transactions.contact_id, transaction_sell_lines.unit_price_inc_tax , GROUP_CONCAT(transaction_sell_lines.quantity SEPARATOR ',') as 
      'number', GROUP_CONCAT(pjt_projects.name SEPARATOR ',') as 'prjects'
       FROM transaction_sell_lines 
       INNER JOIN products on transaction_sell_lines.product_id = products.id 
       INNER JOIN transactions ON transactions.id = transaction_sell_lines.transaction_id 
       INNER JOIN pjt_projects on pjt_projects.id = transactions.pjt_project_id 
       GROUP BY transaction_sell_lines.product_id
       HAVING transactions.contact_id = $draft->contact_id";

        $rows = DB::select(DB::raw($res));
        $rows = json_decode(json_encode($rows));



for ($i=0; $i < count($rows); $i++) {

            $arr = [];
            $prjects = explode(',',$rows[$i]->prjects);
            $numbers = explode(',',$rows[$i]->number);
            for ($j=0; $j <count($prjects) ; $j++) {
                if($arr){
                    $new = collect($arr)->pluck('name');
                    if(in_array($prjects[$j] , $new->toArray())){
                        for ($k=0; $k <count($arr) ; $k++) {
                            if($arr[$k]['name'] == $prjects[$j]){
                                $arr[$k]['number'] += $numbers[$j];
                                $arr[$k]['number'] = number_format($arr[$k]['number'],4);
                                break;
                            }
                        }
                        break; 
                    }
                    
                }

                array_push($arr , ['name' => $prjects[$j] , 'number' => $numbers[$j]]);

                
            }
            $rows[$i]->prjects = $arr;
            $rows[$i]->totalnumber = array_sum($numbers);
            $rows[$i]->totalpricy = array_sum($numbers) * $rows[$i]->unit_price_inc_tax;
        }



        return view('drafts.show',compact('draft','title','rows','total','projects'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Draft $draft)
    {
       
        $title = "Drafts";

        $business_id = request()->session()->get('user.business_id');

        $contact_id = $draft->contact_id;                              // get Contact Id From Request.
        $card_ids= json_decode($draft->cards_id);
        $cards = Transaction::where('business_id',$business_id)->where('contact_id',$contact_id)->get();



        return view('drafts.edit',compact('draft','title','cards','card_ids'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Draft $draft)
    {

        $draft->update([

            'name' => $request->name,
            'status' => "not_approve",
            'contact_id' => $request->contact_id,
            'cards_id' => json_encode($request->category)
        ]);

        return redirect()->route('drafts.show',$draft->id);

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }




    public function print($draft)
    {

        $draft = Draft::find($draft);

        $title = "Draft: ".$draft->name;   // title page.

        $total = 0;

        $projects = DB::select(DB::raw("SELECT pjt_projects.name  FROM pjt_projects"));

        $projects = json_decode(json_encode($projects));



        for ($i=0; $i < count($projects); $i++)
        {
            $projects[$i]->all = 0;
        }


        $res = "SELECT products.name ,transactions.contact_id, transaction_sell_lines.unit_price_inc_tax , GROUP_CONCAT(transaction_sell_lines.quantity SEPARATOR ',') as 
      'number', GROUP_CONCAT(pjt_projects.name SEPARATOR ',') as 'prjects'
       FROM transaction_sell_lines 
       INNER JOIN products on transaction_sell_lines.product_id = products.id 
       INNER JOIN transactions ON transactions.id = transaction_sell_lines.transaction_id 
       INNER JOIN pjt_projects on pjt_projects.id = transactions.pjt_project_id 
       GROUP BY transaction_sell_lines.product_id
       HAVING transactions.contact_id = {$draft->contact_id}";

        $rows = DB::select(DB::raw($res));
        $rows = json_decode(json_encode($rows));



        for ($i=0; $i < count($rows); $i++) {

            $arr = [];
            $prjects = explode(',',$rows[$i]->prjects);
            $numbers = explode(',',$rows[$i]->number);
            for ($j=0; $j <count($prjects) ; $j++) {
                if($arr){
                    $new = collect($arr)->pluck('name');
                    if(in_array($prjects[$j] , $new->toArray())){
                        for ($k=0; $k <count($arr) ; $k++) {
                            if($arr[$k]['name'] == $prjects[$j]){
                                $arr[$k]['number'] += $numbers[$j];
                                $arr[$k]['number'] = number_format($arr[$k]['number'],4);
                                break;
                            }
                        }
                        break;
                    }

                }

                array_push($arr , ['name' => $prjects[$j] , 'number' => $numbers[$j]]);


            }
            $rows[$i]->prjects = $arr;
            $rows[$i]->totalnumber = array_sum($numbers);
            $rows[$i]->totalpricy = array_sum($numbers) * $rows[$i]->unit_price_inc_tax;
        }

        $pdf = PDF::loadView('drafts.print', compact('draft','title','rows','total','projects'));

        $filename = "draft_".$draft->id.".pdf";   // Rename file

        return $pdf->stream($filename);

    }



}
